// ---------------------------------------------------------------------------
// Bei der Entwicklung einer LED-Helligkeitsregelung musste diese Gleichung
// gelöst werden: (1 - exp(x * 1023)) / (1 - exp(x * 511)) - 255 / 63 = 0
//
// 2018-05-10, Tilman Küpper (tilman.kuepper@hm.edu)
// ---------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

double f(double x);
double nullst(double x0, double x1, double epsilon);

// Gewünschte Genauigkeit
#define EPSILON 1e-10

// ---------------------------------------------------------------------------
// Hauptprogramm.
// ---------------------------------------------------------------------------
int main(void)
{
    // Nullstellensuche mittels Bisektionsverfahren; es werden die Startwerte
    // für die Nullstellensuche sowie die gewünschte Genauigkeit übergeben.
    double x = nullst(-0.1, 0.2, EPSILON);
    printf("Nullstelle bei x = %.10f\n", x);
    return 0;
}

// ---------------------------------------------------------------------------
// Definition der Funktion, deren Nullstelle gesucht wird.
// ---------------------------------------------------------------------------
double f(double x)
{
    return (1.0 - exp(x * 1023.0)) / (1.0 - exp(x * 511.0)) - 255.0 / 63.0;
}

// ---------------------------------------------------------------------------
// Nullstellensuche mittels Bisektionsverfahren;
// siehe auch: https://en.wikipedia.org/wiki/Bisection_method
// ---------------------------------------------------------------------------
double nullst(double x0, double x1, double epsilon)
{
    if(f(x0) * f(x1) >= 0)
    {
        printf("nullst(): f(x0) und f(x1) haben gleiches Vorzeichen!\n");
        exit(1);  // *** ABBRUCH DES PROGRAMMS ***
    }

    while(1 == 1)
    {
        double x_neu = (x0 + x1) / 2.0;
        double f_x0 = f(x0), f_xneu = f(x_neu);

        if(fabs(f_xneu) < epsilon)
            return x_neu;

        if(f_x0 * f_xneu >= 0)
            x0 = x_neu;
        else
            x1 = x_neu;
    }
}
